[PATCH 23/27] lib: provide notmuch_config_path

Subject: [PATCH 23/27] lib: provide notmuch_config_path

Date: Thu, 4 Mar 2021 09:01:35 -0400

To: notmuch@notmuchmail.org

Cc: David Bremner

From: David Bremner


Since the library searches in several locations for a config file, the
caller does not know which of these is chosen in the usual case of
passing NULL as a config file. This changes provides an API for the
caller to retrieve the name of the config file chosen. It will be
tested in a following commit.
---
 lib/config.cc          |  5 +++++
 lib/database-private.h |  3 +++
 lib/notmuch.h          | 16 +++++++++++++++-
 lib/open.cc            | 36 +++++++++++++++++++++---------------
 4 files changed, 44 insertions(+), 16 deletions(-)

diff --git a/lib/config.cc b/lib/config.cc
index 855816c1..8aabba5e 100644
--- a/lib/config.cc
+++ b/lib/config.cc
@@ -627,6 +627,11 @@ notmuch_config_get (notmuch_database_t *notmuch, notmuch_config_key_t key) {
     return _notmuch_string_map_get (notmuch->config, _notmuch_config_key_to_string (key));
 }
 
+const char *
+notmuch_config_path (notmuch_database_t *notmuch) {
+    return notmuch->config_path;
+}
+
 notmuch_status_t
 notmuch_config_set (notmuch_database_t *notmuch, notmuch_config_key_t key, const char *val) {
 
diff --git a/lib/database-private.h b/lib/database-private.h
index 2900382d..0d12ec1e 100644
--- a/lib/database-private.h
+++ b/lib/database-private.h
@@ -192,6 +192,9 @@ struct _notmuch_database {
     /* Path to actual database */
     const char *xapian_path;
 
+    /* Path to config loaded, if any */
+    const char *config_path;
+
     int atomic_nesting;
     /* true if changes have been made in this atomic section */
     bool atomic_dirty;
diff --git a/lib/open.cc b/lib/open.cc
index 3c88ff80..95bfadd0 100644
--- a/lib/open.cc
+++ b/lib/open.cc
@@ -113,12 +113,12 @@ _choose_dir (notmuch_database_t *notmuch,
 }
 
 static notmuch_status_t
-_load_key_file (const char *path,
+_load_key_file (notmuch_database_t *notmuch,
+		const char *path,
 		const char *profile,
 		GKeyFile **key_file)
 {
     notmuch_status_t status = NOTMUCH_STATUS_SUCCESS;
-    void *local = talloc_new (NULL);
 
     if (path && EMPTY_STRING (path))
 	goto DONE;
@@ -126,11 +126,13 @@ _load_key_file (const char *path,
     if (! path)
 	path = getenv ("NOTMUCH_CONFIG");
 
-    if (! path) {
-	const char *dir = _xdg_dir (local, "XDG_CONFIG_HOME", ".config", profile);
+    if (path)
+	path = talloc_strdup (notmuch, path);
+    else {
+	const char *dir = _xdg_dir (notmuch, "XDG_CONFIG_HOME", ".config", profile);
 
 	if (dir) {
-	    path = talloc_asprintf (local, "%s/config", dir);
+	    path = talloc_asprintf (notmuch, "%s/config", dir);
 	    if (access (path, R_OK) !=0)
 		path = NULL;
 	}
@@ -139,13 +141,13 @@ _load_key_file (const char *path,
     if (! path) {
 	const char *home = getenv ("HOME");
 
-	path = talloc_asprintf (local, "%s/.notmuch-config", home);
+	path = talloc_asprintf (notmuch, "%s/.notmuch-config", home);
 
 	if (! profile)
 	    profile = getenv ("NOTMUCH_PROFILE");
 
 	if (profile)
-	    path = talloc_asprintf (local, "%s.%s", path, profile);
+	    path = talloc_asprintf (notmuch, "%s.%s", path, profile);
     }
 
     *key_file = g_key_file_new ();
@@ -154,7 +156,9 @@ _load_key_file (const char *path,
     }
 
 DONE:
-    talloc_free (local);
+    if (path)
+	notmuch->config_path = path;
+
     return status;
 }
 
@@ -229,6 +233,7 @@ notmuch_database_t * _alloc_notmuch() {
     notmuch->exception_reported = false;
     notmuch->status_string = NULL;
     notmuch->writable_xapian_db = NULL;
+    notmuch->config_path = NULL;
     notmuch->atomic_nesting = 0;
     notmuch->view = 1;
     return notmuch;
@@ -480,7 +485,7 @@ notmuch_database_open_with_config (const char *database_path,
 	goto DONE;
     }
 
-    status =_load_key_file (config_path, profile, &key_file);
+    status =_load_key_file (notmuch, config_path, profile, &key_file);
     if (status) {
 	message = strdup ("Error: cannot load config file.\n");
 	goto DONE;
@@ -573,7 +578,7 @@ notmuch_database_create_with_config (const char *database_path,
 	goto DONE;
     }
 
-    status =_load_key_file (config_path, profile, &key_file);
+    status =_load_key_file (notmuch, config_path, profile, &key_file);
     if (status) {
 	message = strdup ("Error: cannot load config file.\n");
 	goto DONE;
@@ -760,9 +765,10 @@ notmuch_database_load_config (const char *database_path,
 	goto DONE;
     }
 
-    status =_load_key_file (config_path, profile, &key_file);
+    status =_load_key_file (notmuch, config_path, profile, &key_file);
     switch (status) {
     case NOTMUCH_STATUS_SUCCESS:
+	break;
     case NOTMUCH_STATUS_NO_CONFIG:
 	warning = status;
 	break;
@@ -776,8 +782,8 @@ notmuch_database_load_config (const char *database_path,
     switch (status) {
     case NOTMUCH_STATUS_NO_DATABASE:
     case NOTMUCH_STATUS_SUCCESS:
-	if (! status2)
-	    status2 = status;
+	if (! warning)
+	    warning = status;
 	break;
     default:
 	goto DONE;
@@ -789,8 +795,8 @@ notmuch_database_load_config (const char *database_path,
 	switch (status) {
 	case NOTMUCH_STATUS_NO_DATABASE:
 	case NOTMUCH_STATUS_SUCCESS:
-	    if (! status2)
-		status2 = status;
+	    if (! warning)
+		warning = status;
 	    break;
 	default:
 	    goto DONE;
diff --git a/lib/notmuch.h b/lib/notmuch.h
index 2172d8c8..37c42b1a 100644
--- a/lib/notmuch.h
+++ b/lib/notmuch.h
@@ -454,7 +454,12 @@ notmuch_database_open_with_config (const char *database_path,
  *
  * For description of arguments, @see notmuch_database_open_with_config
  *
  * @retval NOTMUCH_STATUS_SUCCESS: Successfully loaded (some) configuration.
  *
  * @retval NOTMUCH_STATUS_OUT_OF_MEMORY: Out of memory.
  *
@@ -2682,6 +2687,15 @@ notmuch_status_t
 notmuch_config_get_bool (notmuch_database_t *notmuch,
 			 notmuch_config_key_t key,
 			 notmuch_bool_t *val);
+
+/**
+ * return the path of the config file loaded, if any
+ *
+ * @retval NULL if no config file was loaded
+ */
+const char *
+notmuch_config_path (notmuch_database_t *notmuch);
+
 /**
  * get the current default indexing options for a given database.
  *
@@ -454,7 +454,12 @@ notmuch_database_open_with_config (const char *database_path,
  * For description of arguments, @see notmuch_database_open_with_config
  *
- * @retval NOTMUCH_STATUS_SUCCESS: Successfully loaded (some) configuration.
+ * @retval NOTMUCH_STATUS_SUCCESS: Successfully loaded configuration.
+ *
+ * @retval NOTMUCH_STATUS_NO_CONFIG: No config file was loaded. Not fatal.
+ *
+ * @retval NOTMUCH_STATUS_NO_DATABASE: No config information was
+ *	loaded from a database. Not fatal.
  *
  * @retval NOTMUCH_STATUS_OUT_OF_MEMORY: Out of memory.
-- 
2.30.1
_______________________________________________
notmuch mailing list -- notmuch@notmuchmail.org
To unsubscribe send an email to notmuch-leave@notmuchmail.org

Thread: